/******************************************************************************
This test-nl.pro file is aimed as an example of solution for the GetDP version
0.85 distribution Magnetostatic 2D demo where a non linear material substitutes 
the linear core.

It was setup with the help of the GetDP list archives and people over the GetDP 
mailing list, specially Johan Gyselinck. Thanks everybody. 

The idea is to explore the simpler Picard scheme and the more involved Newton
scheme with a iterative loop to deal with the non linearity. This is done both
for the scalar potential and the vector potential formulations.

As with the original demo, there are no currents, the inductors are treated as
air regions. The steel magnetization data was originated somewere in the 
literature as for a "generic 1020" steel. It was artificially extended to 
incorporate very high values for H so that data would be available to test 
extreme cases. This can be easily adapted for other non linear materials. The
test.geo geometry file for Gmsh and the test.msh mesh file are the same as for
the distribution.

There are four possible resolutions in this test-nl.pro file:

1) Vector magnetic potential with Newton scheme:
getdp test-nl -msh test.msh -solve MagSta_a_nl -pos a_nl -v 1
generates a_nl.pos, h_a_nl.pos, b_a_nl.pos

2) Scalar magnetic potential with Newton scheme:
getdp test-nl -msh test.msh -solve MagSta_phi_nl -pos phi_nl -v 1
generates phi_nl.pos, h_phi_nl.pos, b_phi_nl.pos

3) Vector magnetic potential with Picard scheme:
getdp test-nl -msh test.msh -solve MagSta_a_pic -pos a_pic -v 1
generates a_pic.pos, h_a_pic.pos, b_a_pic.pos

4) Scalar magnetic potential with Newton scheme:
getdp test-nl -msh test.msh -solve MagSta_phi_pic -pos phi_pic -v 1
generates phi_pic.pos, h_phi_pic.pos, b_phi_pic.pos

Results from the various resolutions are compared in the file Results.pdf or
Results.ps. File GenSteel.inc has data for non linear material.

#### Warning: ####  I'm a beginner, so use this at your own risk. Comments
and remarks from the list are welcome.
 
Janvrot IVM, Nov 2002
janvrot@infolink.com.br

******************************************************************************/

/* Regions on the geometry file:
  Air     102 
  AirInf  101 
  Core    106 - steel in this example
  Gap     103 - air in this example
  IndP1   111 - air in this example
  IndP2   112 - air in this example
  IndS1   121 - air in this example
  IndS2   122 - air in this example
  Magnet  104 
*/

// Parameters:
  hcMag =  920000.;      // same as for original example
  //hcMag = 500000.;       // test
  Rint = 0.2 ;           // From geometry file, needed by 
  Rext = 0.3 ;           //          VolSphShell Jacobian
  NL_NbrMaxIter = 100;   // Max nbr iterations for iteractive loop
  NL_Eps = 1.e-6;        // Convergence criterium for iteractive loop
  NL_Relax = 1.;         // Relaxation Factor for iteractive loop
  Include "GenSteel.inc" // Data for non linear material

Group {
  Steel     = Region[ 106 ];
  Air       = Region[ {102, 103, 111, 112, 121, 122} ];
  AirInf    = Region[ 101 ];
  Magnet    = Region[ 104 ];
  
  Domain_S  = Region[ {} ] ;  // no currents
  NonSteel  = Region[ {Air, AirInf, Magnet, Domain_S} ];
  NonMagnet = Region[ {Air, AirInf, Steel, Domain_S} ];
  Domain    = Region[ {NonSteel, Steel} ];
  
  Dirichlet_a   = Region[ {1000} ];       // infinity boundary
  Dirichlet_phi = Region[ {1000,1001} ];  // symetry (1000) and infinity boundary
  
}

Function {
  mu0 = 4.e-7 * Pi ;
  muMagnet = 1.1 * mu0;
  mu[ Region[ {Air, AirInf} ] ] = mu0;
  mu[ Magnet ] = muMagnet;
  hc[ Magnet ] = Vector[0., hcMag, 0.];
  js [ Domain_S ] = Vector[0, 0, 0] ;  // no currents
  nu [ Region[ {Air, AirInf}] ]  = 1. / mu0 ;
  nu [ Magnet ] = 1. / muMagnet ;
  nu [ Domain_S ] = 1. / mu0 ;
  
  /* No need to change this if you change materials */
  
  // This is for vector magnetic potential formulation with a Newton scheme
  Steel_nu_b2  = ListAlt[Steel_b2, Steel_nu] ;
  nu_Steel[] = InterpolationAkima[$1]{List[Steel_nu_b2]} ;
  dnudb2_Steel[] = dInterpolationAkima[$1]{List[Steel_nu_b2]} ;
  h_Steel[] = nu_Steel[(SquNorm[$1])] * $1 ;
  dhdb_Steel[] = TensorDiag[1,1,1] * nu_Steel[SquNorm[$1]] + 2*dnudb2_Steel[SquNorm[$1]] * SquDyadicProduct[$1]  ;
  h[Steel] = h_Steel[$1]; dhdb[Steel] = dhdb_Steel[$1];
  
  // This is for scalar magnetic potential formulation with a Newton scheme
  Steel_mu_h2  = ListAlt[Steel_h2, Steel_mu] ;
  mu_Steel[] = InterpolationAkima[$1]{List[Steel_mu_h2]} ;
  dmudh2_Steel[] = dInterpolationAkima[$1]{List[Steel_mu_h2]} ;
  b_Steel[] = mu_Steel[(SquNorm[$1#3])] * #3 ;
  dbdh_Steel[] = TensorDiag[1,1,1] * mu_Steel[SquNorm[$1#1]#2] + 2*dmudh2_Steel[#2] * SquDyadicProduct[#1]  ;
  b[Steel] = b_Steel[$1]; dbdh[Steel] = dbdh_Steel[$1];

  /* This is for vector magnetic potential formulation with a Picard scheme */
  nu[Steel] = InterpolationLinear[Norm[$1]]{List[nu_steel_couples]};
  //nu[Steel] = InterpolationAkima[Norm[$1]]{List[nu_steel_couples]};
  
  /* This is for scalar magnetic potential formulation with a Picard scheme */
  mu[Steel] = InterpolationLinear[Norm[$1]]{List[mu_steel_couples]};
  //mu[Steel] = InterpolationAkima[Norm[$1]]{List[mu_steel_couples]};
}

Constraint {
  { Name a ;
    Case {
      { Region Dirichlet_a ; Value 0. ; }
    }
  }

  { Name phi ;
    Case {
      { Region Dirichlet_phi ; Value 0. ; }
    }
  }
}

Jacobian {
  { Name MyJac;
    Case {
      { Region AirInf ; Jacobian VolSphShell{Rint, Rext} ; }
      { Region All; Jacobian Vol; }
    }
  }
}

Integration {
  { Name MyInt;
    Case {
      { Type Gauss;
        Case {
          { GeoElement Tetrahedron; NumberOfPoints 4; }
          { GeoElement Triangle;    NumberOfPoints 4; }
          { GeoElement Quadrangle;  NumberOfPoints  4; }
        }
      }
    }
  }
}

FunctionSpace {

  { Name Hcurl_a ; Type Form1P ;
    BasisFunction {
      { Name se ; NameOfCoef ae ; Function BF_PerpendicularEdge ;
        Support Domain ; Entity NodesOf[ All ] ; }
    }
    Constraint {
      { NameOfCoef ae ; EntityType NodesOf ; NameOfConstraint a ; }
    }
  }
  
  { Name Hgrad_phi; Type Form0;
    BasisFunction {
      { Name sn; NameOfCoef phin; Function BF_Node;
        Support Domain; Entity NodesOf[ All ]; }
    }
    Constraint {
      { NameOfCoef phin ; EntityType NodesOf ; NameOfConstraint phi ; }
    }
  }
}

Formulation {
  { Name MagSta_a_nl ; Type FemEquation ;
    Quantity {
      { Name a  ; Type Local ; NameOfSpace Hcurl_a ; }
    }
    Equation {

      Galerkin { [ nu[] * Dof{d a} , {d a} ] ; 
                 In NonSteel ; Jacobian MyJac ; Integration MyInt ; }

      Galerkin { JacNL[ dhdb[ {d a} ] * Dof{d a} , {d a} ] ; 
                 In Steel ; Jacobian MyJac ; Integration MyInt ; }
                                     
      Galerkin { [ h[ {d a} ] , {d a} ]  ; 
                 In Steel ; Jacobian MyJac ; Integration MyInt ; }

      Galerkin { [ hc[] , {d a} ] ; 
                 In Magnet ; Jacobian MyJac ; Integration MyInt ; }

      Galerkin { [ -js[] , {a} ] ; 
                 In Domain_S ; Jacobian MyJac ; Integration MyInt ; }
    }
  }
  
  { Name MagSta_phi_nl; Type FemEquation;
    Quantity {
      { Name phi; Type Local; NameOfSpace Hgrad_phi; }
    }
    Equation {
      Galerkin { [ - mu[] * Dof{d phi} , {d phi} ];
                 In NonSteel; Jacobian MyJac; Integration MyInt; }
      
      Galerkin { JacNL[ - dbdh[ {d phi} ] * Dof{d phi} , {d phi} ] ; 
                 In Steel ; Jacobian MyJac ; Integration MyInt ; }

      Galerkin { [ b[ - {d phi} ] , {d phi} ]  ; 
                In Steel ; Jacobian MyJac ; Integration MyInt ; }

      Galerkin { [ - mu[] * hc[] , {d phi} ];
                 In Magnet; Jacobian MyJac; Integration MyInt; }
    }
  }
  
  { Name MagSta_a_pic ; Type FemEquation ;
    Quantity {
      { Name a  ; Type Local ; NameOfSpace Hcurl_a ; }
    }
    Equation {
      Galerkin { [ nu[ {d a} ] * Dof{d a} , {d a} ] ; 
                 In Domain ; Jacobian MyJac ; Integration MyInt ; }

      Galerkin { [ hc[] , {d a} ] ; 
                 In Magnet ; Jacobian MyJac ; Integration MyInt ; }

      Galerkin { [ -js[] , {a} ] ; 
                 In Domain_S ; Jacobian MyJac ; Integration MyInt ; }
    }
  }
  
  { Name MagSta_phi_pic; Type FemEquation;
    Quantity {
      { Name phi; Type Local; NameOfSpace Hgrad_phi; }
    }
    Equation {
      Galerkin { [ - mu[ {d phi} ] * Dof{d phi} , {d phi} ];
                 In Domain; Jacobian MyJac; Integration MyInt; }
      
      Galerkin { [ - mu[] * hc[] , {d phi} ];
                 In Magnet; Jacobian MyJac; Integration MyInt; }
    }
  }
}


Resolution {
  { Name MagSta_a_nl ;
    System {
      { Name A ; NameOfFormulation MagSta_a_nl ; }
    }
    Operation {
      InitSolution [A];
      IterativeLoop[NL_NbrMaxIter, NL_Eps, NL_Relax] {
        GenerateJac[A]; SolveJac[A];
      }
      SaveSolution[A];
    }
  }
  
  { Name MagSta_phi_nl;
    System {
      { Name A; NameOfFormulation MagSta_phi_nl; }
    }
    Operation {
      InitSolution [A];
      IterativeLoop[NL_NbrMaxIter, NL_Eps, NL_Relax] {
        GenerateJac[A]; SolveJac[A];
      }
      SaveSolution[A];
    }
  }
  
  { Name MagSta_a_pic ;
    System {
      { Name A ; NameOfFormulation MagSta_a_pic ; }
    }
    Operation {
      InitSolution [A];
      IterativeLoop[NL_NbrMaxIter, NL_Eps, NL_Relax] {
        GenerateJac[A]; SolveJac[A];
      }
      SaveSolution[A];
    }
  }
  
  { Name MagSta_phi_pic;
    System {
      { Name A; NameOfFormulation MagSta_phi_pic; }
    }
    Operation {
      InitSolution [A];
      IterativeLoop[NL_NbrMaxIter, NL_Eps, NL_Relax] {
        GenerateJac[A]; SolveJac[A];
      }
      SaveSolution[A];
    }
  }
}

PostProcessing {
  { Name MagSta_a_nl ; NameOfFormulation MagSta_a_nl ;
    Quantity {
      { Name a ; Value { Local { [ CompZ[{a}] ]   ; In Domain   ; Jacobian MyJac ; } } }
      { Name b ; Value { Local { [ {d a} ]        ; In Domain   ; Jacobian MyJac ; } } }
      { Name a ; Value { Local { [ {a} ]          ; In Domain   ; Jacobian MyJac ; } } }
      { Name h ; Value { Local { [ nu[] * {d a} ] ; In NonSteel ; Jacobian MyJac ; }
                         Local { [ h[ {d a} ] ]   ; In Steel    ; Jacobian MyJac ; }
                         Local { [ hc[] ]         ; In Magnet   ; Jacobian MyJac ; } } }
    }
  }
  
  { Name MagSta_phi_nl; NameOfFormulation MagSta_phi_nl;
    Quantity {
      { Name phi; Value { Local { [ {phi} ]      ; In Domain ; Jacobian MyJac;} } }
      { Name h;   Value { Local { [ - {d phi} ]  ; In Domain ; Jacobian MyJac;} } }
      { Name b;   Value
           {
               Local { [ - mu[] * {d phi} ]      ; In NonSteel ; Jacobian MyJac; }
               Local { [ b[ - {d phi} ] ]        ; In Steel    ; Jacobian MyJac; }
               Local { [ - mu[] * hc[] ]         ; In Magnet   ; Jacobian MyJac; }
           }
      }
    }
  }
  
  { Name MagSta_a_pic ; NameOfFormulation MagSta_a_pic ;
    Quantity {
      { Name a ; Value { Local { [ CompZ[ {a} ] ]        ; In Domain ; Jacobian MyJac ; } } }
      { Name b ; Value { Local { [ {d a} ]               ; In Domain ; Jacobian MyJac ; } } }
      { Name a ; Value { Local { [ {a} ]                 ; In Domain ; Jacobian MyJac ; } } }
      { Name h ; Value { Local { [ nu[ {d a} ] * {d a} ] ; In Domain ; Jacobian MyJac ; }
                         Local { [ hc[] ]                ; In Magnet ; Jacobian MyJac ; } } }
    }
  }
  
  { Name MagSta_phi_pic; NameOfFormulation MagSta_phi_pic;
    Quantity {
      { Name phi; Value { Local { [ {phi} ]     ; In Domain; Jacobian MyJac;} } }
      { Name h;   Value { Local { [ - {d phi} ] ; In Domain; Jacobian MyJac;} } }
      { Name b;   Value
           {
               //Local { [ - mu[] * {d phi} ]        ; In NonSteel ; Jacobian MyJac; }
               Local { [ - mu[ {d phi} ] * {d phi} ] ; In Domain   ; Jacobian MyJac; }
               Local { [ - mu[] * hc[] ]             ; In Magnet   ; Jacobian MyJac; }
           }
      }
    }
  }
}

PostOperation {
  { Name a_nl; NameOfPostProcessing MagSta_a_nl;
    Operation {
      // Basic values
      Print[ a,   OnElementsOf Domain, File "a_nl.pos"] ;
      Print[ b,   OnElementsOf Domain, File "b_a_nl.pos", Depth 0 ] ;
      Print[ h,   OnElementsOf Domain, File "h_a_nl.pos", Depth 0 ] ;
    }
  }
  
  { Name phi_nl; NameOfPostProcessing MagSta_phi_nl;
    Operation {
      // Basic values
      Print[ phi, OnElementsOf Domain, File "phi_nl.pos"] ;
      Print[ b,   OnElementsOf Domain, File "b_phi_nl.pos", Depth 0 ] ;
      Print[ h,   OnElementsOf Domain, File "h_phi_nl.pos", Depth 0 ] ;
    }
  }

  { Name a_pic; NameOfPostProcessing MagSta_a_pic;
    Operation {
      // Basic values
      Print[ a,   OnElementsOf Domain, File "a_pic.pos"] ;
      Print[ b,   OnElementsOf Domain, File "b_a_pic.pos", Depth 0 ] ;
      Print[ h,   OnElementsOf Domain, File "h_a_pic.pos", Depth 0 ] ;    
    }
  }

  { Name phi_pic; NameOfPostProcessing MagSta_phi_pic;
    Operation {
      // Basic values
      Print[ phi, OnElementsOf Domain, File "phi_pic.pos"] ;
      Print[ b,   OnElementsOf Domain, File "b_phi_pic.pos", Depth 0 ] ;
      Print[ h,   OnElementsOf Domain, File "h_phi_pic.pos", Depth 0 ] ;   
    }
  }

}

